/*
 *
 * Copyright (c) 2014-2021 Jean-Pierre Andre
 *
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (in the main directory of the NTFS-3G
 * distribution in the file COPYING); if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef EA_H
#define EA_H

int ntfs_ea_check_wsldev(ntfs_inode *ni, dev_t *rdevp);

int ntfs_ea_set_wsl_not_symlink(ntfs_inode *ni, mode_t mode, dev_t dev);

int ntfs_get_ntfs_ea(ntfs_inode *ni, char *value, size_t size);

int ntfs_set_ntfs_ea(ntfs_inode *ni, const char *value, size_t size, int flags);

int ntfs_remove_ntfs_ea(ntfs_inode *ni);

#endif /* EA_H */
